@@ -1,7 +1,7 @@
 %%%-------------------------------------------------------------------
 %%% File    : yaws_pam.erl
 %%% Author  :  <klacke@hyber.org>
-%%% Description : 
+%%% Description :
 %%%
 %%% Created : 20 Dec 2005 by  <klacke@hyber.org>
 %%%-------------------------------------------------------------------
@@ -15,7 +15,7 @@
         ]).
 
 %% gen_server callbacks
--export([init/1, handle_call/3, handle_cast/2, handle_info/2, 
+-export([init/1, handle_call/3, handle_cast/2, handle_info/2,
          terminate/2, code_change/3]).
 -export([auth/2,
          close/1]).
@@ -71,7 +71,7 @@ close(Handle) ->
 %%--------------------------------------------------------------------
 
 init([]) ->
-    {Srv, Act,Sess} = 
+    {Srv, Act,Sess} =
         {okundef(application:get_env(pam_service)),
          okundef(application:get_env(pam_use_acct)),
          okundef(application:get_env(pam_use_sess))},
@@ -86,7 +86,7 @@ init([SRV, Act, Sess]) ->
     %% in yaws
 
     M1 = case Act of
-             undefined -> 
+             undefined ->
                  "";
              true ->
                  "A";
@@ -107,7 +107,7 @@ init([SRV, Act, Sess]) ->
     Mode = M1 ++ M2,
 
     %% we're not starting the portprogram now, it's done
-    %% on demand. 
+    %% on demand.
     {ok, #state{i = 0,
                 mode = Mode,
                 srv = SRV,
@@ -134,7 +134,7 @@ okundef(undefined) ->
 handle_call({auth, User, Password}, From, State0) ->
     State = ensure_port(State0),
     I = integer_to_list(State#state.i),
-    port_command(State#state.port, [$a, User, 0, Password, 0, 
+    port_command(State#state.port, [$a, User, 0, Password, 0,
                                     State#state.mode,0, I, 0]),
     Ref = erlang:monitor(process, element(1, From)),
     U = #user{i = State#state.i,
@@ -157,8 +157,7 @@ handle_call({close, Sid}, _From, State = #state{port = Port}) ->
     port_command(Port, [$c, integer_to_list(Sid), 0]),
 
     {reply, ok, State#state{
-                  sids = lists:keydelete(Sid, #user.i, State#state.sids)},
-     ?TO}.
+                  sids = lists:keydelete(Sid, #user.i, State#state.sids)},?TO}.
 
 
 %%--------------------------------------------------------------------
@@ -178,10 +177,10 @@ handle_cast(_Msg, State) ->
 %%          {noreply, State, Timeout} |
 %%          {stop, Reason, State}            (terminate/2 is called)
 %%--------------------------------------------------------------------
-handle_info(timeout, State) when 
-State#state.port /= undefined,
-State#state.sids == [],
-State#state.reqs == [] ->
+handle_info(timeout, State) when
+      State#state.port /= undefined,
+      State#state.sids == [],
+      State#state.reqs == [] ->
     unlink(State#state.port),
     port_close(State#state.port),
     {noreply, State#state{port = undefined}};
@@ -198,11 +197,11 @@ handle_info({'EXIT', Port, _}, State = #state{port = Port}) ->
                           reqs = [],
                           port = undefined
                          }};
-handle_info({'DOWN', MonitorRef, _Type, _Object, _Info}, State) 
-when State#state.port /= undefined ->
+handle_info({'DOWN', MonitorRef, _Type, _Object, _Info}, State)
+  when State#state.port /= undefined ->
     case lists:keysearch(MonitorRef, #user.ref, State#state.sids) of
         {value, U} ->
-            port_command(State#state.port, 
+            port_command(State#state.port,
                          [$c, integer_to_list(U#user.i), 0]),
             S2 = lists:keydelete(MonitorRef, #user.ref, State#state.sids),
             {noreply, State#state{sids = S2}, ?TO};
@@ -224,7 +223,7 @@ handle_info({_Port, {data, Str}}, State) ->
                     no ->
                         State#state.sids
                 end,
-            {noreply, 
+            {noreply,
              State#state{reqs = lists:keydelete(I,#user.i,State#state.reqs),
                          sids = R}, ?TO};
         _Other ->
@@ -256,7 +255,7 @@ code_change(_OldVsn, State, _Extra) ->
 
 
 reply(From, Sid, ["yes"]) ->
-    gen_server:reply(From, {yes, Sid}), 
+    gen_server:reply(From, {yes, Sid}),
     yes;
 reply(From, _Sid, ["no", What |Reason ]) ->
     gen_server:reply(From, {no, {What, fsp(Reason)}}),
@@ -265,11 +264,11 @@ reply(From, _Sid, ["no", What |Reason ]) ->
 fsp([]) -> [];
 fsp([X]) -> X;
 fsp([H|T]) -> H ++ " " ++ fsp(T).
-    
-    
+
+
 
 ensure_port(S = #state{port = undefined, srv = Srv}) ->
-    Prg0 = filename:dirname(code:which(?MODULE)) ++ 
+    Prg0 = filename:dirname(code:which(?MODULE)) ++
         "/../priv/epam ",
 
     Prg = Prg0 ++ Srv,
