@@ -26,7 +26,9 @@
                 type,   %% client | server
                 r_req,  %% if'we're server, what req method are we processing
                 r_host, %% and value of Host: for the cli request
-                httpconnection="keep-alive", %% Do we need to keep the connection open ("keep-alive" | "close")  
+                httpconnection="keep-alive", %% Do we need to keep the
+                                             %% connection open
+                                             %% ("keep-alive" | "close")
                 state}).%% various depending on mode, ......
 
 %% MREMOND: TODO: Check if redirection works properly (this is
@@ -60,29 +62,29 @@ init(CliSock, ARG, _DecPath, _QueryPart, {Prefix, URL}, N) ->
             %%  MREMOND: TODO: Refactor.
             yaws:gen_tcp_send(Ssock, [ReqStr, "\r\n", Hstr, "\r\n"]),
             RemainingData = if
-                       N /= 0 ->
-                           %% Send data to the server:
-                           ClientData=ARG#arg.clidata,
-                           yaws:gen_tcp_send(Ssock,ClientData),
+                                N /= 0 ->
+                                    %% Send data to the server:
+                                    ClientData=ARG#arg.clidata,
+                                    yaws:gen_tcp_send(Ssock,ClientData),
                                     N - size(ClientData);
-                       true ->
-                           %% MREMOND: Check this (If N == 0, it might
-                           %% be because, we do not know the size of
-                           %% the data to receive. For now we will get
-                           %% back in http mode, expecting headers
-                           %% (new request)
-                           0
-                   end,
+                                true ->
+                                    %% MREMOND: Check this (If N == 0, it might
+                                    %% be because, we do not know the size of
+                                    %% the data to receive. For now we will get
+                                    %% back in http mode, expecting headers
+                                    %% (new request)
+                                    0
+                            end,
 
             Cli2 = case RemainingData of
-                       0      -> 
+                       0      ->
                            %% Sockmode will be changed in the ploop function
                            Cli#psock{mode = expectheaders, state = undefined};
                        _Other -> Cli
                    end,
-            
-            Srv = #psock{s = Ssock, 
-                         prefix = Prefix, 
+
+            Srv = #psock{s = Ssock,
+                         prefix = Prefix,
                          url = URL,
                          r_req = (ARG#arg.req)#http_request.method,
                          r_host = (ARG#arg.headers)#headers.host,
@@ -92,23 +94,24 @@ init(CliSock, ARG, _DecPath, _QueryPart, {Prefix, URL}, N) ->
                                             true ->
                                                  yaws:to_lower(KeepAlive)
                                          end
-                        },     
-            %% Need to check if we could close the connection after server answer
+                        },
+            %% Need to check if we could close the connection after serveranswer
             %% Now we _must_ spawn a process here, because we
             %% can't use {active, once} due to the inefficencies
             %% that would occur with chunked encodings
-            P1 = proc_lib:spawn_link(?MODULE, ploop, [Cli2, Srv, GC, SC, self()]),
+            P1 = proc_lib:spawn_link(?MODULE, ploop,
+                                     [Cli2, Srv, GC, SC, self()]),
             ?Debug("Client=~p, Srv=~p", [P1, self()]),
             ploop(Srv, Cli2, GC, SC, P1);
         _ERR ->
             yaws:outh_set_dyn_headers(ARG#arg.req, ARG#arg.headers,
                                       #urltype{}),
             yaws_server:deliver_dyn_part(
-              CliSock,  
+              CliSock,
               0, "404",
               0,
               ARG, no_UT_defined,
-              fun(A)->(SC#sconf.errormod_404):out404(A,get(gc),get(sc)) 
+              fun(A)->(SC#sconf.errormod_404):out404(A,get(gc),get(sc))
               end,
               fun(A)->yaws_server:finish_up_dyn_file(A, CliSock)
               end
@@ -136,6 +139,10 @@ rewrite_path(Req, Pref) ->
     New.
 
 
+strip_prefix("","") ->
+    "/";
+strip_prefix(P,"") ->
+    P;
 strip_prefix(P,"/") ->
     P;
 strip_prefix([H|T1],[H|T2]) ->
@@ -144,7 +151,7 @@ strip_prefix([H|T1],[H|T2]) ->
 
 
 %% Once we have read the headers, what comes after
-%% the headers, 
+%% the headers,
 %% This is applicable both for cli and srv sockets
 
 sockmode(H,Req,Psock) ->
@@ -160,9 +167,9 @@ s_sockmode(H,Resp,Psock) ->
             %% we're replying to a HEAD
             %% no body
             Psock#psock{mode = expectheaders, state = undefined};
-        
-        true ->
-            case lists:member(Resp#http_response.status, 
+
+       true ->
+            case lists:member(Resp#http_response.status,
                               [100,204,205,304,406]) of
                 true ->
                     %% no body, illegal
@@ -173,7 +180,7 @@ s_sockmode(H,Resp,Psock) ->
     end.
 
 cont_len_check(H,Psock) when H#headers.transfer_encoding == "chunked" ->
-	Psock#psock{mode = expectchunked, state = init};
+    Psock#psock{mode = expectchunked, state = init};
 cont_len_check(H,Psock) ->
     case H#headers.content_length of
         undefined ->
@@ -189,7 +196,7 @@ cont_len_check(H,Psock) ->
         List when is_list(List) ->
             Psock#psock{mode = len,
                         state = list_to_integer(List)}
-    
+
     end.
 
 
@@ -234,13 +241,13 @@ ploop(From0, To, Pid) ->
             case yaws:http_get_headers(From#psock.s, get(ssl)) of
                 {R, H0} ->
                     ?Debug("R = ~p~n",[R]),
-                    RStr = 
+                    RStr =
                         if
                             %% FIXME handle bad_request here
                             is_record(R, http_response) ->
                                 yaws_api:reformat_response(R);
                             is_record(R, http_request) ->
-                                Pid ! {cli2srv, R#http_request.method, 
+                                Pid ! {cli2srv, R#http_request.method,
                                        H0#headers.host},
                                 yaws_api:reformat_request(
                                   rewrite_path(R, From#psock.prefix))
@@ -260,11 +267,11 @@ ploop(From0, To, Pid) ->
                     ok=yaws:eat_crnl(From#psock.s, get(ssl)),
                     yaws:gen_tcp_send(TS,["0\r\n\r\n"]),
                     ?Debug("SEND final 0 ",[]),
-                    ploop_keepalive(From#psock{mode = expectheaders, 
-                                     state = undefined},To, Pid);
+                    ploop_keepalive(From#psock{mode = expectheaders,
+                                               state = undefined},To, Pid);
                true ->
-                    ploop(From#psock{mode = chunk, 
-                                    state = N},To, Pid)
+                    ploop(From#psock{mode = chunk,
+                                     state = N},To, Pid)
             end;
         chunk ->
             CG = yaws:get_chunk(From#psock.s,From#psock.state, 0, get(ssl)),
@@ -276,7 +283,7 @@ ploop(From0, To, Pid) ->
                              state = undefined}, To, Pid);
         len when From#psock.state == 0 ->
             ploop_keepalive(From#psock{mode = expectheaders,
-                             state = undefined},To, Pid);
+                                       state = undefined},To, Pid);
         len ->
             case yaws:do_recv(From#psock.s, From#psock.state, get(ssl)) of
                 {ok, Bin} ->
@@ -286,7 +293,7 @@ ploop(From0, To, Pid) ->
                     ploop(From#psock{state = From#psock.state - SZ},
                           To, Pid);
                 _Rsn ->
-                    ?Debug("Failed to read :~p~n", [_Rsn]),  
+                    ?Debug("Failed to read :~p~n", [_Rsn]),
                     exit(normal)
             end;
         undefined ->
@@ -310,9 +317,9 @@ ploop_keepalive(From, To, Pid) ->
     %% TODO: We should get this value from the config file
     case check_server_keepalive() of
         false -> done; %% Close the connection: Server config do not
-                       %%  allow proxy keep-alive
+        %%  allow proxy keep-alive
         true -> ploop(From, To, Pid) %% Try keeping the connection
-                                     %% alive => Wait for headers
+                %% alive => Wait for headers
     end.
 
 %% TODO: Get proxy keepalive value in SC record
@@ -342,7 +349,7 @@ rewrite_headers(PS, H) when PS#psock.type == client ->
 
 
 
-%% And on the way from the server to the client we 
+%% And on the way from the server to the client we
 %% need to rewrite the Location header, and the
 %% Set-Cookie header
 
@@ -360,12 +367,12 @@ rewrite_headers(PS, H) when PS#psock.type == server ->
                       LocUrl#url.port == ProxyUrl#url.port,
                       LocUrl#url.scheme == ProxyUrl#url.scheme ->
                           rewrite_loc_url(LocUrl, PS);
-                      
+
                       element(1, LocUrl) == 'EXIT' ->
                           rewrite_loc_rel(PS, H#headers.location);
                       true ->
-                          ?Debug("Not rew ~p~n~p~n", 
-                              [LocUrl, ProxyUrl]),
+                          ?Debug("Not rew ~p~n~p~n",
+                                 [LocUrl, ProxyUrl]),
                           H#headers.location
                   end
           end,
@@ -395,7 +402,7 @@ rewrite_loc_rel(PS, Loc) ->
     RedirHost = yaws:redirect_host(SC, PS#psock.r_host),
     [Scheme, RedirHost,Loc].
 
-    
+
 
 slash_append("/", [$/|T]) ->
     [$/|T];
@@ -407,15 +414,15 @@ slash_append([], T) ->
     [$/|T];
 slash_append([H|T], X) ->
     [H | slash_append(T,X)].
-        
 
 
 
 
-        
 
 
-        
+
+
+
 
 
 
