@@ -21,6 +21,12 @@ start() ->
     streamcontent_test(),
     sendfile_get(),
     json_test(),
+    post_test(),
+    expires_test(),
+    reentrant_test(),
+    php_handler_test(),
+    arg_rewrite_test(),
+    shaper_test(),
     ibrowse:stop().
 
 
@@ -267,6 +273,8 @@ streamcontent_test() ->
     gen_tcp:close(Sock),
     ok.
 
+-define(JSON_URI, "http://localhost:8005/jsontest").
+
 json_test() ->
     io:format("json_test\n",[]),
     io:format("  param array1\n", []),
@@ -285,6 +293,10 @@ json_test() ->
                        {struct, [{"jsonrpc", "2.0"},
                                  {"result", -19},
                                  {"id", 2}]}),
+    inets:start(),
+    io:format("  encode/decode\n", []),
+    ?line {ok,{response,[19]}} = jsonrpc:call(?JSON_URI, [],
+                                              {call, "subtract", [42, 23]}),
     io:format("  param obj1\n", []),
     ?line ok = do_json({struct, [{"jsonrpc", "2.0"},
                                  {"method", "subtract"},
@@ -483,7 +495,7 @@ json_send(Req) ->
 json_send(Req, encode) ->
     json_send(json2:encode(Req), no_encode);
 json_send(Req, no_encode) ->
-    Uri = "http://localhost:8005/jsontest",
+    Uri = ?JSON_URI,
     ReqHdrs = [{content_type, "application/json"}],
     ibrowse:send_req(Uri, ReqHdrs, post, Req).
 
@@ -506,6 +518,160 @@ recv_hdrs(Sock, Len) ->
             {error, {"unexpected message", Other}}
     end.
 
+
+%% partial_post_size = 2048000
+post_test() ->
+    io:format("post_test\n",[]),
+    small_post(),
+    large_post(),
+    small_chunked_post(),
+    large_chunked_post(),
+    ok.
+
+small_post() ->
+    io:format("  small post\n",[]),
+    {ok, Bin} = file:read_file("../../www/1000.txt"),
+    Sz = size(Bin),
+    Uri = "http://localhost:8006/posttest/" ++ integer_to_list(Sz),
+    Hdrs = [{content_length, Sz}, {content_type, "binary/octet-stream"}],
+    ?line {ok, "200", _, _} = ibrowse:send_req(Uri, Hdrs, post, Bin, []),
+    ok.
+
+large_post() ->
+    io:format("  large post\n",[]),
+    {ok, Bin} = file:read_file("../../www/10000.txt"),
+    Sz = size(Bin),
+    Uri = "http://localhost:8006/posttest/" ++ integer_to_list(Sz),
+    Hdrs = [{content_length, Sz}, {content_type, "binary/octet-stream"}],
+    ?line {ok, "200", _, _} = ibrowse:send_req(Uri, Hdrs, post, Bin, []),
+    ok.
+
+small_chunked_post() ->
+    io:format("  small chunked post\n",[]),
+    {ok, Bin} = file:read_file("../../www/3000.txt"),
+    Sz = size(Bin),
+    Uri = "http://localhost:8006/posttest/chunked/" ++ integer_to_list(Sz),
+    Hdrs = [{content_type, "binary/octet-stream"}],
+    Opts = [{transfer_encoding, {chunked, 1000*1000}}],
+    ?line {ok, "200", _, _} = ibrowse:send_req(Uri, Hdrs, post, Bin, Opts),
+    ok.
+
+large_chunked_post() ->
+    io:format("  large chunked post\n",[]),
+    {ok, Bin} = file:read_file("../../www/10000.txt"),
+    Sz = size(Bin),
+    Uri = "http://localhost:8006/posttest/chunked/" ++ integer_to_list(Sz),
+    Hdrs = [{content_type, "binary/octet-stream"}],
+
+    %% size of chunk _IS_NOT_ a multiple of partial_post_size
+    Opts1 = [{transfer_encoding, {chunked, 4000*1000}}],
+    ?line {ok, "200", _, _} = ibrowse:send_req(Uri, Hdrs, post, Bin, Opts1),
+
+    %% size of chunk _IS_ a multiple of partial_post_size
+    Opts2 = [{transfer_encoding, {chunked, 4000*1024}}],
+    ?line {ok, "200", _, _} = ibrowse:send_req(Uri, Hdrs, post, Bin, Opts2),
+    ok.
+
+
+expires_test() ->
+    io:format("expires_test\n", []),
+    Uri = "http://localhost:8006/icons/yaws.gif",
+    ?line {ok, "200", Hdrs, _} = ibrowse:send_req(Uri, [], get),
+
+    %% Retrieve max-age value to test Expires header
+    ?line "max-age=" ++ Rest = proplists:get_value("Cache-Control", Hdrs),
+    ?line Secs = list_to_integer(Rest),
+
+    %% Convert Date and Expires into datetime()
+    ?line Date = proplists:get_value("Date", Hdrs),
+    ?line Expires = proplists:get_value("Expires", Hdrs),
+    Date_DT = httpd_util:convert_request_date(Date),
+    Expires_DT = httpd_util:convert_request_date(Expires),
+
+    %% Check if Expires value is equal to "Date + max-age"
+    Val1 = calendar:datetime_to_gregorian_seconds(Date_DT) + Secs,
+    Val2 = calendar:datetime_to_gregorian_seconds(Expires_DT),
+    ?line Val1 = Val2,
+    ok.
+
+
+reentrant_test() ->
+    io:format("reentrant_test\n", []),
+    reentrant_test_status(),
+    reentrant_test_delayed_headers(),
+    ok.
+
+reentrant_test_status() ->
+    io:format("  status code\n", []),
+    Uri = "http://localhost:8006/reentranttest/status",
+    ?line {ok, "201", _, _} = ibrowse:send_req(Uri, [], post, <<"blob">>, []),
+    ok.
+
+reentrant_test_delayed_headers() ->
+    io:format("  delayed headers\n", []),
+    Uri = "http://localhost:8006/reentranttest/delayed_headers",
+    ?line {ok, "200", Hdrs, _} = ibrowse:send_req(Uri, [], get),
+    ?line "no-cache" = proplists:get_value("Cache-Control", Hdrs),
+    ?line "static-tag" = proplists:get_value("Etag", Hdrs),
+    ?line "true" = proplists:get_value("X-Delayed-Header", Hdrs),
+    ok.
+
+
+
+php_handler_test() ->
+    io:format("php_handler_test\n", []),
+    Uri = "http://localhost:8006/test.php",
+    {ok, Binary} = file:read_file("../../www/test.php"),
+    Content = binary_to_list(Binary),
+    ?line {ok, "200", _, Content} = ibrowse:send_req(Uri, [], get),
+    ok.
+
+
+arg_rewrite_test() ->
+    io:format("arg_rewrite_test\n", []),
+    arg_rewrite_test_rewrite(),
+    arg_rewrite_test_redirect(),
+    arg_rewrite_test_response(),
+    ok.
+
+arg_rewrite_test_rewrite() ->
+    io:format("  rewrite\n", []),
+    Uri = "http://localhost:8006/rewrite",
+    ?line {ok, "200", Hdrs, _} = ibrowse:send_req(Uri, [], get),
+    ?line "image/gif" = proplists:get_value("Content-Type", Hdrs),
+    {ok, FI} = file:read_file_info("../../www/icons/yaws.gif"),
+    Etag = yaws:make_etag(FI),
+    ?line Etag = proplists:get_value("Etag", Hdrs),
+    ok.
+
+arg_rewrite_test_redirect() ->
+    io:format("  redirect\n", []),
+    Uri = "http://localhost:8006/redirect",
+    ?line {ok, "301", Hdrs, _} = ibrowse:send_req(Uri, [], get),
+    ?line "http://www.yakaz.com" = proplists:get_value("Location", Hdrs),
+    ok.
+
+
+arg_rewrite_test_response() ->
+    io:format("  response\n", []),
+    Uri = "http://localhost:8006/response",
+    ?line {ok, "200", Hdrs, Content} = ibrowse:send_req(Uri, [], get),
+    ?line "text/plain" = proplists:get_value("Content-Type", Hdrs),
+    ?line "Hello, World!" = Content,
+    ok.
+
+
+
+shaper_test() ->
+    io:format("shaper_test\n", []),
+    Uri = "http://localhost:8007/",
+    ?line {ok, "200", _, _} = ibrowse:send_req(Uri, [], get),
+    ?line {ok, "200", _, _} = ibrowse:send_req(Uri, [], get),
+    ?line {ok, "200", _, _} = ibrowse:send_req(Uri, [], get),
+    ?line {ok, "503", _, _} = ibrowse:send_req(Uri, [], get),
+    ok.
+
+
 %% used for appmod tests
 %%
 out(_A) ->
